LIBRARY = $(shell basename $(shell pwd))

SHAREDLIB = $(LIBRARY).so

prefix = $(RPM_BUILD_ROOT)

ifeq ($(MODE),C)

OBJ_EXT=c
CFLAGS += -I../../include $(RPM_OPT_FLAGS) -fpic $(shell guile-config compile)
COMPILLER=$(CC)
COMPILLER_FLAGS=$(CFLAGS)
else

OBJ_EXT=cc
CXXFLAGS += -I../../include $(RPM_OPT_FLAGS) -fpic $(shell guile-config compile)
COMPILLER=$(CXX)
COMPILLER_FLAGS=$(CXXFLAGS)
endif

LDLIBS += $(shell guile-config link)

LINKER = $(COMPILLER) $(LDFLAGS) $(TARGET_ARCH)

LIB_SRC += $(wildcard *.$(OBJ_EXT))

#use sort to remove possible duplicates
LIB_OBJ = $(sort $(LIB_SRC:%.$(OBJ_EXT)=%.o))
LIB_DEP = $(sort $(LIB_SRC:%.$(OBJ_EXT)=%.d))

TARGETS = $(SHAREDLIB)

.PHONY: all install clean

all: $(TARGETS)

$(SHAREDLIB): $(LIB_OBJ)
	echo $(LIB_SRC)
	$(LINKER) -shared $+ $(LDLIBS) $(OUTPUT_OPTION)

clean:
	$(RM) $(TARGETS) $(LIB_OBJ) $(LIB_DEP) $(SONAME) $(LIB_CLEAN) core *~

install: all
	install -pm0644 -D $(SHAREDLIB) $(libdir)/guile/2.2/extensions/$(SHAREDLIB)

# We need dependencies only if goal isn't "clean"
ifneq ($(MAKECMDGOALS),clean)

%.d:	%.$(OBJ_EXT)
	@echo Making dependences for $<
	@$(SHELL) -ec "$(COMPILLER) -MM $(COMPILLER_FLAGS) $< | sed 's|\($*\)\.o[ :]*|\1.o $@ : |g' > $@; [ -s $@ ] || $(RM) $@"

ifneq ($(LIB_DEP),)
-include $(LIB_DEP)
endif
endif
