(define-module (alterator systemd)
  :use-module (ice-9 popen)
  :use-module (ice-9 rdelim)
  :export (sd-listen-fds
           systemctl-simple))

; args: [unset-flag]
; returns: list of the socket file descriptor numbers passed to the process
; in the course of the systemd socket activation contract.
(define (sd-listen-fds . args)
  (let* ((listen-pid-str (getenv "LISTEN_PID"))
         (listen-pid (and listen-pid-str (string->number listen-pid-str)))
         (listen-fds-str (getenv "LISTEN_FDS"))
         (listen-fds (and listen-fds-str (string->number listen-fds-str)))
         (unset-fds (and (not (null? args)) (car args))))
    (if unset-fds
      (begin
        (unsetenv "LISTEN_PID")
        (unsetenv "LISTEN_FDS")))
    (if (and listen-fds listen-pid (= listen-pid (getpid)))
      (let loop ((fd 3) (fds '()))
        (if (< fd (+ 3 listen-fds))
          (begin
            (fcntl fd F_SETFD
                   (logior FD_CLOEXEC
                           (fcntl fd F_GETFD)))
            (loop (+ fd 1) (append fds (list fd))))
          fds))
      '())))

; args: service-base-name, i.e. with no .socket or .service suffix;
;       [command-name] -- one of the 'status, 'activate, 'deactivate,
;       'socket.
; returns: status pair for the 'status command, result text, or simply
;          #t or #f.
(define (systemctl-simple service-base-name . args)
  (let* ((command (or (and (not (null? args)) (car args)) 'status))
         (p (open-pipe* OPEN_READ "alterator-systemctl-helper" service-base-name (symbol->string command)))
         (res (read-line p))
         (ret (status:exit-val (close-pipe p))))
    (and res
         (case command
           ((status) (and (not (eof-object? res)) (not (string-null? res))
                          (let ((reslist (string-split res #\-)))
                            (cons (string->symbol (car reslist))
                                  (string->symbol (cadr reslist))))))
           (else (or (and (not (eof-object? res)) (not (string-null? res)) res)
                     (and ret (= ret 0))))))))
